@media print {

  * {
    text-shadow: none !important;
    color: #000 !important; // Black prints faster: h5bp.com/s
    background: transparent !important;
    box-shadow: none !important;
  }

  a,
  a:visited {
    text-decoration: underline;
  }

  a[href]:after {
    content: " (" attr(href) ")";
  }

  abbr[title]:after {
    content: " (" attr(title) ")";
  }

  // Don't show links for images, or javascript/internal links
  a[href^="javascript:"]:after,
  a[href^="#"]:after {
    content: "";
  }

  pre,
  blockquote {
    border: 1px solid #999 !important;
    page-break-inside: avoid;
  }

  thead {
    display: table-header-group; // h5bp.com/t
  }

  tr,
  img {
    page-break-inside: avoid;
  }

  img {
    max-width: 100% !important;
  }

  p,
  h2,
  h3 {
    orphans: 3;
    widows: 3;
  }

  h2,
  h3 {
    page-break-after: avoid;
  }

  // Chrome (OSX) fix for https://github.com/twbs/bootstrap/issues/11245
  // Once fixed, we can just straight up remove this.
  select {
    background: #fff !important;
  }

  // Bootstrap components
  .navbar {
    display: none;
  }
  .table {
    td,
    th {
      background-color: #fff !important;
    }
  }
  .btn,
  .dropup > .btn {
    > .caret {
      border-top-color: #000 !important;
    }
  }
  .label {
    border: 1px solid #000;
  }

  .table {
    border-collapse: collapse !important;
  }
  .table-bordered {
    th,
    td {
      border: 1px solid #ddd !important;
    }
  }

  header, nav, footer, #portal-searchbox, #portal-personaltools, #portal-breadcrumbs, .global-footer, #edit-bar {
    display: none;
  }
 .toolbar-initialized .wrap, body.castle-toolbar-active #visual-wrapper, body.template-folder_contents.castle-toolbar-active #main-content-container, body.template-theming-controlpanel.castle-toolbar-active {
      margin: 5px !important;
      width: 100% !important;
      padding: 0px !important;
  }

// Not shipped with plone

@page {
  margin: 2cm;
}
body {
   font-size: 12px !important;
}

h1,h2,h3,h4,h5,h6 {
/*Prevent headings from being printed at the bottom of the page   */
page-break-after: avoid;
}

article {
/*Always start new articles on a new page*/
page-break-before: always;
}

ul {
/*Prevent images from being split up*/
page-break-inside: avoid;
}
.featured-content-backgroundimage {
  max-width: 100%;
  height: auto !important;
  .info-text {
    position: static !important;
    background: transparent !important;
    h3 {
      color: black !important;
      font-size: 110% !important;
      text-shadow: initial !important;
    }
    p {
      color: black !important;
      font-size: 100% !important;
      text-shadow: initial !important;
    }
  }
}

img, .focuspoint img {
  max-width: 100% !important;
  min-height: initial !important;
  min-width: initial !important;
  display: initial !important;
}
.focuspoint {
  page-break-inside: avoid !important;
  img {
    position: inherit;
    top: initial !important;
    left: initial !important;
    max-width: 100% !important;
  }
}

.mosaic-tile .focuspoint {
  height: auto !important;
}
}
