tr.historyComment td{padding:0 !important}tr.historyComment td p{margin:0;padding:0;margin-left:15px;padding-left:10px;border-left:1px solid #ccc;border-bottom:1px solid #ccc}.revert-btn{font-size:.7em !important;padding:1px 5px !important;height:22px}.revert-form{display:inline}