.survey-invite {
  text-align: center;
  padding: 10px;

  img {
    max-width:350px;
    margin: 0 auto;
  }

  .survey-api-title {
    margin-bottom: 15px;
    padding:10px;
    color: #404040;
    font-size:  30px;
  }

  .survey-api-desc {
    color: #404040;
    margin-bottom: 15px;
    font-size: 20px;
  }
}

.survey-disclaimer {
  text-align: center;
  font-size: 0.8em;
  color: #888888;
}

.plone-modal-footer > .survey-disclaimer {
  margin-top: 0px;
}

.survey-no-api {
  font-weight: 700;
  color: #C13333;
}
