/* overridding plone's base less file to not include icons */
@fontsPrefix: '@{staticPath}/fonts/';

@import url("@{mockup-patterns-select2}");
@import url("@{mockup-patterns-pickadate}");
@import url("@{mockup-patterns-autotoc}");
@import url("@{mockup-patterns-modal}");
@import url("@{mockup-patterns-markspeciallinks}");
@import (less) "@{tinymce-default-styles}";

@import "@{bowerPath}bootstrap/less/variables.less";
@icon-font-path: "@{bowerPath}bootstrap/dist/fonts/";

/* tinymce added image styles */
.image-left {
  float: left;
  margin: 0.5em 1em 0.5em 0;
}
.image-right {
  float: right;
  margin: 0.5em;
}
.image-inline {
  float: none;
}

/* icons-off styles to hide icons from
   site setting */
body#visual-portal-wrapper.pat-plone.icons-off .outer-wrapper [class*="contenttype-"]:before {
  display:none;
}

/* error message styles */
body.template-default_error_message #content-core p {
  white-space: pre;
}

div#disclaimerOverlay {
  background-color: rgba(0,0,0,0.5);
  margin: auto;
  width: 100%;
  position: absolute;
  top: 100px;
  left: 500px;
  z-index: 9001;
}

/* slide down animation */
div#disclaimerOverlay div {
  animation: 2s slide-down;
  margin-top: 0%;
}

@keyframes slide-down {
  from {
    margin-top: -100%;
  }

  to {
    margin-top: 0%;
  }
}

/* first time visit disclaimer */
div#disclaimerDiv {
  overflow: hidden;
  position: absolute;
  visibility: visible;
  padding: 10px;
  background-color: white;
  min-width: 200px;
  max-width: 300px;
  border-radius: 3px;
  box-shadow: 0 1px 10px rgba(0,0,0,.4);
  -webkit-transition: opacity 0.1s ease-out;
     -moz-transition: opacity 0.1s ease-out;
      -ms-transition: opacity 0.1s ease-out;
       -o-transition: opacity 0.1s ease-out;
          transition: opacity 0.1s ease-out;
}

#closeButton {
  position: absolute;
  overflow: visible;
  display: inline-block;
  padding: 0.3em 0.8em;
  border: 1px solid #d4d4d4;
  margin: 5px;
  text-decoration: none;
  text-shadow: 1px 1px 0 #fff;
  font: 11px/normal sans-serif;
  color: #333;
  white-space: nowrap;
  cursor: pointer;
  outline: none;
  right: 0;
  bottom: 0;
  background-color: #ececec;
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#f4f4f4), to(#ececec));
  background-image: -moz-linear-gradient(#f4f4f4, #ececec);
  background-image: -o-linear-gradient(#f4f4f4, #ececec);
  background-image: linear-gradient(#f4f4f4, #ececec);
  -webkit-background-clip: padding;
  -moz-background-clip: padding;
  -o-background-clip: padding-box;
  /*background-clip: padding-box;*/ /* commented out due to Opera 11.10 bug */
  -webkit-border-radius: 0.2em;
  -moz-border-radius: 0.2em;
  border-radius: 0.2em;
  /* IE hacks */
  zoom: 1;
  *display: inline;
  margin-top: 10px;
}

#closeButton:hover {
  border-color: #bcbcbc;
  text-decoration: none;
  box-shadow: 0px 1px 1px #e3e3e3;
}

#closeButton:focus,
#closeButton:active {
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#ececec), to(#f4f4f4));
  background-image: -moz-linear-gradient(#ececec, #f4f4f4);
  background-image: -o-linear-gradient(#ececec, #f4f4f4);
  background-image: linear-gradient(#ececec, #f4f4f4);
}
