body.secure-login-template {
  background: #fafafa;
}
.castle-login-header {
  text-align: center;
  margin: 25px 0;
}
.castle-login-header img {
  max-width: 400px;
  max-height: 400px;
}
#secure-login {
  margin-top: 20px;
}
.auth-description {
  font-size: 14px;
  color: #767676;
  font-weight: 300;
}
.castle-secure-login-container,
.login-exception-container {
  background: #fff;
  border: 1px solid #dedede;
  padding: 10px 20px 30px;
  border-radius: 4px;
  box-shadow: 0px 2px 5px 0px rgba(0, 0, 0, 0.15);
  max-width: 600px;
  margin: 0 auto;
  overflow: hidden;
}
.castle-secure-login-container .form-control,
.login-exception-container .form-control {
  background-color: #fcfcfc;
  margin-bottom: 20px;
  border-radius: 2px;
}
.castle-secure-login-container .form-control:focus,
.login-exception-container .form-control:focus {
  box-shadow: none;
}
.castle-secure-login-container .auth-title,
.login-exception-container .auth-title {
  text-align: center;
  margin: 10px 0 30px;
  font-size: 24px;
  font-weight: 400;
}
.castle-secure-login-container .radio,
.login-exception-container .radio {
  margin-bottom: 15px;
}
.castle-secure-login-container .alert,
.login-exception-container .alert {
  clear: both;
}
.castle-secure-login-buttons {
  margin: 30px 0;
}
.castle-secure-login-buttons .btn {
  margin-right: 10px;
  border-radius: 2px;
  color: #333;
  font-size: 14px;
}
.castle-secure-login-buttons .btn-primary {
  color: #fff;
  background-color: #337ab7;
  border-color: #2e6da4;
}
.castle-secure-login-buttons .btn-primary:hover {
  color: #fff;
  background-color: #286090;
  border-color: #204d74;
}
#login-forgotten-password {
  max-width: 600px;
  margin: 20px auto;
  text-align: center;
}
.castle-secure-login-forms-container {
  width: 9000px;
  margin-left: 0;
  transition: margin-left 0.25s ease-in-out;
  -moz-transition: margin-left 0.25s ease-in-out;
  -webkit-transition: margin-left 0.25s ease-in-out;
}
.castle-secure-login-forms-container > div {
  width: 560px;
  padding: 10px 20px 0;
  float: left;
}
.castle-secure-login-forms-container.auth-code-sent {
  margin-left: -560px;
}
.castle-secure-login-forms-container.code-authorized {
  margin-left: -1120px;
}
.castle-secure-login-forms-container.change-password {
  margin-left: -1680px;
}
.castle-secure-login-forms-container.country-blocked {
  margin-left: -2240px;
}
.castle-secure-login-forms-container.country-block-requested {
  margin-left: -2800px;
}
.castle-secure-login-login-provider {
  list-style: none;
  margin: 0;
  padding: 0;
}
.castle-secure-login-login-provider > li {
  display: block;
  width: 100%;
  margin: 0;
  padding: 0;
}
.castle-secure-login-login-provider > li a {
  display: block;
  color: white;
  margin: 2px;
  padding: 5px;
  border: 1px solid #dedede;
  border-radius: 4px;
  box-shadow: 0px 2px 5px 0px rgba(0, 0, 0, 0.15);
  background-color: #72abd5;
}
.castle-secure-login-login-provider > li a:hover {
  text-decoration: none;
  border-color: #2e6da4;
  background-color: #337ab7;
}
.castle-secure-login-login-provider > li img {
  max-width: 20px;
  max-height: 20px;
  padding-right: 5px;
}
